@@ -24,7 +24,23 @@ if [ $has_wayland_backend = "yes" ]; then
         cppargs="$cppargs -DCLUTTER_WINDOWING_WAYLAND"
 fi
 
+echo "1..1"
+echo "# Start of abicheck"
+
 cpp -P ${cppargs} ${srcdir:-.}/clutter.symbols | sed -e '/^$/d' -e 's/ G_GNUC.*$//' -e 's/ PRIVATE//' -e 's/ DATA//' | sort > expected-abi
 
 nm -D -g --defined-only .libs/libclutter-1.0.so | cut -d ' ' -f 3 | egrep -v '^(__bss_start|_edata|_end)' | sort > actual-abi
-diff -u expected-abi actual-abi && rm -f expected-abi actual-abi
+
+diff -u expected-abi actual-abi > diff-abi
+
+if [ $? = 0 ]; then
+        echo "ok 1 expected abi"
+        rm -f diff-abi
+else
+        echo "not ok 1 expected abi"
+        echo "# difference in diff-abi"
+fi
+
+rm -f actual-abi expected-abi
+
+echo "# End of abicheck"
