@@ -81,7 +81,7 @@ clutter_input_device_dispose (GObject *gobject)
 {
   ClutterInputDevice *device = CLUTTER_INPUT_DEVICE (gobject);
 
-  g_free (device->device_name);
+  g_clear_pointer (&device->device_name, g_free);
 
   if (device->associated != NULL)
     {
@@ -93,23 +93,10 @@ clutter_input_device_dispose (GObject *gobject)
       device->associated = NULL;
     }
 
-  if (device->axes != NULL)
-    {
-      g_array_free (device->axes, TRUE);
-      device->axes = NULL;
-    }
-
-  if (device->keys != NULL)
-    {
-      g_array_free (device->keys, TRUE);
-      device->keys = NULL;
-    }
-
-  if (device->touch_sequences_info)
-    {
-      g_hash_table_unref (device->touch_sequences_info);
-      device->touch_sequences_info = NULL;
-    }
+  g_clear_pointer (&device->axes, g_array_unref);
+  g_clear_pointer (&device->keys, g_array_unref);
+  g_clear_pointer (&device->scroll_info, g_array_unref);
+  g_clear_pointer (&device->touch_sequences_info, g_hash_table_unref);
 
   if (device->inv_touch_sequence_actors)
     {
