@@ -165,7 +165,7 @@ clutter_backend_x11_xsettings_notify (const char       *name,
     {
       if (g_strcmp0 (name, CLUTTER_SETTING_X11_NAME (i)) == 0)
         {
-          GValue value = { 0, };
+          GValue value = G_VALUE_INIT;
 
           switch (setting->type)
             {
@@ -1344,11 +1344,8 @@ _clutter_x11_input_device_translate_screen_coord (ClutterInputDevice *device,
     return FALSE;
 
   info = &g_array_index (device->axes, ClutterAxisInfo, index_);
-  if (info->axis != CLUTTER_INPUT_AXIS_X ||
-      info->axis != CLUTTER_INPUT_AXIS_Y)
-    {
-      return FALSE;
-    }
+  if (!(info->axis == CLUTTER_INPUT_AXIS_X || info->axis == CLUTTER_INPUT_AXIS_Y))
+    return FALSE;
 
   width = info->max_value - info->min_value;
 
